# -*- coding: utf-8 -*-
import re

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

import django_filters

class AdvancedSearchModelAdmin(admin.ModelAdmin):

    @property
    def search_filterset_class(self):
        """Return a filterset_class suitable for advanced search
        first look if attribute ``advanced_search_fields`` exists or
        :meth:`get_filterset_class` exists
        """
        fields = getattr(self, 'advanced_search_fields', None)
        if fields:
            meta = type('Meta', (object,), {'fields': fields, 'model' : self.model})
            AdminFilterset = type('Admin%sFilterset' % self.model.__name__, 
                                  (django_filters.FilterSet,),
                                  { 'Meta' : meta, 'fields' : fields})
            return AdminFilterset
        elif hasattr(self, 'get_filterset_class'):
            return self.get_filterset()
        return None

    def lookup_allowed(self, lookup, value):
        return True  ## FIXME: below code should work eather
        if hasattr(self, '_lookup_search'):
            if lookup in self._lookup_names:
                return True
        return super(ExtendibleModelAdmin, self).lookup_allowed(lookup, value)

    def get_changelist(self, request):
        return AdvancedSearchChangeList

    def queryset(self, request):

        if self.search_filterset_class:
            self.search_filterset = self.search_filterset_class(request.GET)
            self.search_form = self.search_filterset.form
            self.search_form.is_valid()
            self._lookup_names = re.findall('name="(?P<name>[^\"]*)"', str(self.search_form))

            return self.search_filterset.qs
        return admin.ModelAdmin.queryset(self, request)
            
class AdvancedSearchChangeList(ChangeList):

    def get_filters(self, request):
        """
        clean self.params settings the values cleaned by the
        search_form, so that dates can be localized
        """

        if hasattr(self.model_admin, 'search_form'):
            search_form = self.model_admin.search_form
            search_form.is_valid()

            if search_form.is_valid():
                for name in self.model_admin._lookup_names:
                    self.params.pop(name, None)

        return ChangeList.get_filters(self, request)
    
